import { getNodeList } from './nodelist';

function createImage(src) {
  return new Promise((resolve, reject) => {
    const image = document.createElement('img');

    image.onload = () => {
      resolve(image);
    };

    image.onerror = (error) => {
      reject(error);
    };

    image.style.cssText += 'position: absolute; visibility: hidden;';
    image.src = src;
  });
}

function tryCatch(tryFunc, catchFunc = () => {}) {
  try {
    return tryFunc();
  } catch (error) {
    return catchFunc(error);
  }
}

function blobObject(uInt8Array, type) {
  const bp = tryCatch(
    () => {
      return new Blob([uInt8Array], {
        type
      });
    },
    (error) => {
      const BlobBuilder = window.BlobBuilder ||
        window.WebKitBlobBuilder ||
        window.MozBlobBuilder ||
        window.MSBlobBuilder;

      if (error.name == 'TypeError' && BlobBuilder) {
        const blobBuilder = new BlobBuilder();
        blobBuilder.append(uInt8Array.buffer);
        return blobBuilder.getBlob(type);
      }

      return new Blob([uInt8Array.buffer], {
        type
      });
    }
  );

  return bp;
}

function canvasImageSize(opts, string) {
  const splitString = string.split(','); //[1].replace(/(=){1,}/g, '');
  const binary = atob(splitString[1].replace(/(=){1,}/g, ''));
  const charArray = [];
  for (let i = 0, iMax = binary.length; i < iMax; i++) {
    charArray[i] = binary.charCodeAt(i);
  }

  const imageBlob = blobObject(new Uint8Array(charArray), splitString[0].split(';')[0].split(':')[1]);

  fileToBase64(opts, imageBlob);

  return `${imageBlob.size / 1000}kb`;
}

function contain(image, width, height) {
  image.style.cssText += `width: ${width}px; height: auto;`;

  if (image.offsetHeight > height) {
    image.style.cssText += `width: auto; height: ${height}px;`;
  }
}

function fill(image, width, height) {
  image.style.cssText += `width: ${width}px; height: auto;`;

  if (image.offsetHeight < height) {
    image.style.cssText += `width: auto; height: ${height}px;`;
  }
}

function resizeImageElement(image, opts) {
  document.body.appendChild(image);

  const params = Object.assign({
    constrain: 'fill'
  }, opts);

  const imageDimensions = {
    width: image.offsetWidth,
    height: image.offsetHeight
  };

  const target = params.target ? getNodeList(params.target)[0] : {};
  const paramDimensions = {
    width: params.width ? params.width : target.clientWidth,
    height: params.height ? params.height : target.clientHeight
  };


  const width = paramDimensions.width || imageDimensions.width;
  const height = paramDimensions.height || imageDimensions.height;

  if (params.constrain === 'contain') {
    contain(image, width, height);
  } else {
    fill(image, width, height);
  }

  document.body.removeChild(image);
}

function getFileFormat(base64String) {
  const stringParts = base64String.split(';base64,');
  const contentType = stringParts[0].split(':')[1];
  const rawImage = window.atob(stringParts[1].replace(/(=){1,}/g, ''));
  const rawImageLength = rawImage.length;

  const arrayBuffer = new Uint8Array(rawImageLength);

  const data = arrayBuffer.reduce((originalData, item, index) => {
    originalData[index] = rawImage.charCodeAt(index);
    return originalData;
  }, arrayBuffer);

  const header = data.subarray(0, 4).reduce((originalHeader, item, index) => {
    return `${originalHeader}${item.toString(16)}`;
  }, '');

  let mimeType;
  switch (header) {
  case '89504e47':
    mimeType = 'image/png';
    break;
  case '47494638':
    mimeType = 'image/gif';
    break;
  case 'ffd8ffe0':
  case 'ffd8ffe1':
  case 'ffd8ffe2':
    mimeType = 'image/jpeg';
    break;
  default:
    mimeType = contentType;
    break;
  }

  return mimeType;
}

function parseFileFormat(inputType) {
  return (inputType.indexOf('image/') === -1 ? `image/${inputType}` : inputType).replace('jpg', 'jpeg');
}

function renderInCanvas(params) {
  const { element } = params;
  const isVideo = element.tagName === 'VIDEO';
  const defaultQuality = 0.92;

  if (!isVideo) {
    document.body.appendChild(element);
  }

  const canvas = document.createElement('canvas');
  const width = params.width || element.videoWidth || element.offsetWidth;
  const height = params.height || element.videoHeight || element.offsetHeight;

  canvas.setAttribute('width', width);
  canvas.setAttribute('height', height);

  const context = canvas.getContext('2d');
  const fileFormat = parseFileFormat(params.fileFormat || params.fileType || 'image/png');

  // context.imageSmoothingEnabled = false;
  context.drawImage(element, 0, 0, width, height);
  const data = canvas.toDataURL(fileFormat, params.quality || defaultQuality);

  if (!isVideo) {
    document.body.removeChild(element);
  }

  const base64String = data.split(',')[1].replace(/(=){1,}/g, '');
  const fileSize = `${(base64String.length * 0.75) / 1000}kb`;

  return {
    canvas,
    fileSize,
    image: data
  };
}

function fileToBase64(opts, file) {
  return new Promise((resolve, reject) => {
    if (['image/jpeg', 'image/jpg', 'image/png', 'image/gif'].indexOf(file.type) > -1) {
      const reader = new FileReader();

      reader.readAsDataURL(file);

      reader.onload = () => {
        const base64String = reader.result.split(',')[1].replace(/(=){1,}/g, '');
        const fileSize = `${(base64String.length * 0.75) / 1000}kb`;
        resolve({
          fileSize,
          fileType: file.type,
          fileName: file.name,
          image: reader.result
        });
      };

      reader.onerror = (error) => {
        window.console.error(`Error reading the file ${file.name}: `, error);
        reject(error);
      };
    } else {
      reject(`invalid image type: ${file.type}`);
    }
  });
}

/**
 * Crops an image (base64 string) that is passed to the function. You must include at least a width or a height or a target. Crop will always centre the image before cropping
 * @param {Object} opts - The parameters required to execute the crop function
 * @param {string} opts.image - A base64 string representation of an image. You can use your own base64 string, or use the base64 string returned from a uploadImage or takePicture call
 * @param {number} [opts.width=undefined] - The final width you want the image to have.
 * @param {number} [opts.height=undefined] - The final height you want the image to have.
 * @param {string|Element|NodeList} [opts.target=undefined] -
 * - A target element that will be used as the basis for resizing the image.
 * - param.target can be a querySelector, a DOM element, or a DOM node list
 * @return {Promise} A promise - The resolved promise will have a response argument containing the result of the request
 */
function crop(opts) {
  return new Promise((resolve, reject) => {
    createImage(opts.image).then(
      (image) => {
        document.body.appendChild(image);

        const imageDimensions = {
          width: image.offsetWidth,
          height: image.offsetHeight
        };

        let width = opts.width || imageDimensions.width;
        let height = opts.height || imageDimensions.height;

        if (opts.target) {
          const container = getNodeList(opts.target)[0];
          width = container.clientWidth;
          height = container.clientHeight;
        }

        const cropX = ((imageDimensions.width - width) / 2);
        const cropY = ((imageDimensions.height - height) / 2);

        const canvas = document.createElement('canvas');
        canvas.setAttribute('width', width);
        canvas.setAttribute('height', height);

        const context = canvas.getContext('2d');
        context.drawImage(
          image,
          cropX,
          cropY,
          width,
          height,
          0,
          0,
          width,
          height
        );

        const data = canvas.toDataURL(opts.fileType);

        document.body.removeChild(image);

        const base64String = data.split(',')[1].replace(/(=){1,}/g, '');
        const fileSize = `${(base64String.length * 0.75) / 1000}kb`;

        resolve(
          Object.assign({},
            opts, {
              canvas,
              fileSize,
              image: data
            }
          )
        );
      },
      (error) => {
        window.console.error('crop failed: ', error);
        reject(error);
      }
    );
  });
}

/**
 * Resizes an image (base64 string) that is passed to the function. You must include at least a width or a height or a target. The aspect ratio of the image will always be maintained, even if the width, height, or target establish dimensions that do not match the aspect ratio of the image. If you need a specific shape, you should run crop after resize
 * @param {Object} opts - The parameters required to execute the resize function
 * @param {string} opts.image - A base64 string representation of an image. You can use your own base64 string, or use the base64 string returned from a uploadImage or takePicture call
 * @param {number} [opts.width=undefined] - The final width you want the image to have.
 * @param {number} [opts.height=undefined] - The final height you want the image to have.
 * @param {string|Element|NodeList} [opts.target=undefined] -
 * - A target element that will be used as the basis for resizing the image.
 * - param.target can be a querySelector, a DOM element, or a DOM node list
 * @param {string} [opts.constrain=fill] -
 * - The way in which you want to constrain the image in the target (container) element. Constrain values can be:
 * - "contain". Make sure the whole image fits within the boundaries of the container. This could mean that the width or height could be smaller than the container, depending on the aspect ratio of the image
 * - "fill". Fill the entire container is filled. This could mean that the width or height could overflow beyond the boundaries of the container, depending on the aspect ratio of the image
 * @return {Promise} A promise - The resolved promise will have a response argument containing the result of the request
 */
function resize(opts) {
  return new Promise((resolve, reject) => {
    createImage(opts.image).then(
      (image) => {
        document.body.appendChild(image);
        document.body.removeChild(image);

        resizeImageElement(image, opts);

        const params = Object.assign({}, opts, {
          element: image
        });

        resolve(Object.assign({}, opts, renderInCanvas(params)));
      },
      (error) => {
        window.console.error('resize failed: ', error);
        reject(error);
      }
    );
  });
}

export {
  fileToBase64,
  resize,
  crop,
  renderInCanvas
};
