import { fileToBase64 } from './image-transform';
import { getNodeList } from './nodelist';
import validateParams from './validate-params';

function convertImage(uploadField, opts, resolve, reject, detect, event) {
  if (uploadField.value && !detect.hasFired) {
    detect.hasFired = true;

    fileToBase64(opts, event.target.files[0]).then(
      (data) => {
        resolve(data);
      },
      (error) => {
        reject(error);
      }
    );
  }
}

function createFileUploader(opts, resolve, reject) {
  const uploadField = document.createElement('input');
  uploadField.setAttribute('type', 'file');
  uploadField.value = null;

  const detect = {
    hasFired: false
  };

  const conversion = convertImage.bind(this, uploadField, opts, resolve, reject, detect);
  uploadField.addEventListener('change', conversion, false);
  uploadField.click();

  const interval = setInterval(function () {
    if (uploadField.value && !detect.hasFired) {
      clearInterval(interval);

      fileToBase64(opts, uploadField.files[0]).then(
        (data) => {
          resolve(data);
        },
        (error) => {
          reject(error);
        }
      );
    } else if (detect.hasFired) {
      clearInterval(interval);
    }
  }, 200);
}

/**
 * Initialises the image uploading functionality, that will kick in once a specified event has been triggered
 * @param {Object} params - The parameters required to execute the uploadImage function
 * @param {string} params.event - The event that will trigger the file upload dialog
 * @param {string|Element|NodeList} params.target - The target element that will receive an event specified in the "event"
 * param.
 * param.target can be a querySelector, a DOM element, or a DOM node list
 * @return {Promise} A promise - The resolved promise will have a response argument containing the result of the request
 * @example
 * imageHandler.uploadImage({
 *  target: '#myButton',
 *  event: 'click'
 * }).then(function (response) {
 *  var fileSize = response.fileSize;
 *  var imageFormat = repsonse.fileType;
 *  var fileName = response.fileName;
 *  var base64String = response.image;
 * });
 */
function uploadImage(params) {
  const props = validateParams('uploadImage', params, []);

  if (!props.valid) {
    window.console.error(props.error);
    return Promise.reject(props.error);
  }

  return new Promise((resolve, reject) => {
    createFileUploader(props, resolve, reject);
  });
}

export {
  uploadImage
};
