import { crop, resize, renderInCanvas } from './image-transform';
import { getNodeList } from './nodelist';
import { tryCatch } from './utils';

function sizeToFit(video, target, stream) {
  return new Promise((resolve) => {
    video.style.cssText += `width: ${target.clientWidth}px; height: auto;`;

    if (video.offsetHeight < target.clientHeight) {
      video.style.cssText += `width: auto; height: ${target.clientHeight}px;`;

      video.addEventListener('loadedmetadata', function (event) {
        video.style.marginLeft = `-${Math.round((video.offsetWidth - target.clientWidth) / 2)}px`;

        resolve({
          video,
          stream
        });
      });
    }
  });
}

function getUserMedia(resolve, reject, opts) {
  navigator.mediaDevices.getUserMedia({
    video: true,
    audio: false
  }).then(
    (stream) => {
      const video = document.createElement('video');

      const targets = getNodeList(opts.videoTarget);
      if (!targets) {
        reject('No valid video targets specified');
      }
      const target = targets[0];

      target.appendChild(video);

      video.srcObject = stream;
      video.play();

      sizeToFit(video, target, stream).then((data) => {
        resolve(data);
      });
    },
    (error) => {
      window.console.error(`An error occured resolving the getUserMedia promise: ${error}`);
      return reject(`An error occured resolving the getUserMedia promise: ${error}`);
    }
  );
}

function startCamera(opts) {
  const hasWebcamSupport = navigator.mediaDevices && navigator.mediaDevices.getUserMedia;
  const params = Object.assign({
    videoTarget: opts.target
  }, opts);

  if (!hasWebcamSupport) {
    return Promise.reject('Your browser does not support getUserMedia (webcam media)');
  }
  if (!params.videoTarget) {
    return Promise.reject('No valid target element specified');
  }

  return new Promise((resolve, reject) => {
    getUserMedia(resolve, reject, params);
  });
}

function takePicture(opts) {
  const params = Object.assign({
    fileType: 'image/png',
    element: opts.video,
    resize: true
  }, opts);

  const result = Object.assign(params, renderInCanvas(params));

  tryCatch(
    () => { params.stream.getTracks()[0].stop(); },
    () => { params.stream.stop(); }
  );
  params.element.parentNode.removeChild(params.element);

  return !params.resize ? Promise.resolve(result) : resize(result).then(data => crop(data));
}

export {
  startCamera,
  takePicture
};
